@@ -18,11 +18,11 @@ package Image::ExifTool::CanonRaw;
 
 use strict;
 use vars qw($VERSION $AUTOLOAD %crwTagFormat);
-use Image::ExifTool qw(:DataAccess);
+use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 use Image::ExifTool::Canon;
 
-$VERSION = '1.49';
+$VERSION = '1.50';
 
 sub WriteCRW($$);
 sub ProcessCanonRaw($$$);
@@ -314,10 +314,13 @@ sub BuildMakerNotes($$$$$$);
     0x1814 => { #3
         Name => 'MeasuredEV',
         Notes => q{
-            this the Canon name for what could better be called MeasuredLV, and is
-            offset by about -5 EV from the calculated LV for most models
+            this is the Canon name for what could better be called MeasuredLV, and
+            should be close to the calculated LightValue for a proper exposure with most
+            models
         },
         Format => 'float',
+        ValueConv => '$val + 5',
+        ValueConvInv => '$val - 5',
     },
     0x1817 => {
         Name => 'FileNumber',
@@ -771,7 +774,7 @@ sub ProcessCanonRaw($$$)
             my $name = $$tagInfo{Name};
             my $newTagTable;
             if ($$subdir{TagTable}) {
-                $newTagTable = Image::ExifTool::GetTagTable($$subdir{TagTable});
+                $newTagTable = GetTagTable($$subdir{TagTable});
                 unless ($newTagTable) {
                     warn "Unknown tag table $$subdir{TagTable}\n";
                     next;
@@ -850,7 +853,7 @@ sub ProcessCRW($$)
     );
 
     # process the raw directory
-    my $rawTagTable = Image::ExifTool::GetTagTable('Image::ExifTool::CanonRaw::Main');
+    my $rawTagTable = GetTagTable('Image::ExifTool::CanonRaw::Main');
     unless ($exifTool->ProcessDirectory(\%dirInfo, $rawTagTable)) {
         $exifTool->Warn('CRW file format error');
     }
@@ -894,7 +897,7 @@ tags.)
 
 =head1 AUTHOR
 
-Copyright 2003-2009, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
