@@ -14,7 +14,7 @@ package Image::ExifTool::NikonCustom;
 use strict;
 use vars qw($VERSION);
 
-$VERSION = '1.00';
+$VERSION = '1.02';
 
 # custom settings for the D80 (encrypted) - ref JD
 %Image::ExifTool::NikonCustom::SettingsD80 = (
@@ -340,7 +340,7 @@ $VERSION = '1.00';
     10.1 => { # CS22-d
         Name => 'RepeatingFlashRate',
         Mask => 0xf0,
-        ValueConv => 'my $v=($val>>4); $v < 10 ? $v + 1 : 10 * ($val - 8)',
+        ValueConv => 'my $v=($val>>4); $v < 10 ? $v + 1 : 10 * ($v - 8)',
         ValueConvInv => 'int(($val <= 10 ? $val - 1 : $val / 10 + 8) + 0.5) << 4',
         PrintConv => '"$val Hz"',
         PrintConvInv => '$val=~/(\d+)/; $1 || 0',
@@ -462,8 +462,8 @@ $VERSION = '1.00';
             0x02 => 'On',
         },
     },
-    16 => { # CS2
-        Name => 'AFAreaMode',
+    16.1 => { # CS2
+        Name => 'AFAreaModeSetting',
         Mask => 0xc0,
         PrintConv => {
             0x00 => 'Single Area',
@@ -546,7 +546,7 @@ $VERSION = '1.00';
             0x07 => '1 s',
         },
     },
-    2 => { # CS15-b
+    2.1 => { # CS15-b
         Name => 'ImageReviewTime',
         Mask => 0x07,
         PrintConv => {
@@ -640,7 +640,7 @@ $VERSION = '1.00';
             0x04 => 'Self-timer',
         },
     },
-    6 => { # CS5
+    6.1 => { # CS5
         Name => 'Metering',
         Mask => 0x03,
         PrintConv => {
@@ -676,7 +676,7 @@ $VERSION = '1.00';
         PrintConv => 'sprintf("%+.1f",$val)',
         PrintConvInv => '$val',
     },
-    10 => { # CS2
+    10.1 => { # CS2
         Name => 'FocusModeSetting',
         # (may differ from FocusMode if lens switch is set to Manual)
         Mask => 0xc0,
@@ -687,7 +687,7 @@ $VERSION = '1.00';
             0xc0 => 'AF-A',
         },
     },
-    11 => { # CS3
+    11.1 => { # CS3
         Name => 'AFAreaModeSetting',
         # (may differ from AFAreaMode for Manual focus)
         Mask => 0x30,
@@ -699,6 +699,498 @@ $VERSION = '1.00';
     }
 );
 
+# D90 custom settings (ref PH)
+%Image::ExifTool::NikonCustom::SettingsD90 = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
+    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    WRITABLE => 1,
+    FIRST_ENTRY => 0,
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
+    NOTES => 'Custom settings for the D90.',
+    # Missing:
+    # CSe2 RepeatingFlashRate (needs verification)
+    #      CommanderInternalFlash, CommanderGroupAMode, CommanderGroupBMode,
+    #      CommanderChannel, CommanderInternalManualOutput,
+    #      CommanderGroupA_ManualOutput, CommanderGroupB_ManualOutput
+    #      CommanderGroupA_TTL-AAComp, CommanderGroupB_TTL-AAComp,
+    # CSe4 AutoBracketSet (some values need verification)
+    # CSf2 OKButton ("Not Used" value needs verification)
+    # CSf5-b CommandDialsChangeMainSub
+    # CSf5-c CommandDialsMenuAndPlayback
+    0.1 => { # CSf1
+        Name => 'LightSwitch',
+        Mask => 0x08,
+        PrintConv => {
+            0x00 => 'LCD Backlight',
+            0x08 => 'LCD Backlight and Shooting Information',
+        },
+    },
+    2.1 => { # CSa1
+        Name => 'AFAreaModeSetting',
+        Mask => 0x60,
+        PrintConv => {
+            0x00 => 'Single Area',
+            0x20 => 'Dynamic Area',
+            0x40 => 'Auto-area',
+            0x60 => '3D-tracking (11 points)',
+        },
+    },
+    2.2 => { # CSa2
+        Name => 'CenterFocusPoint',
+        Mask => 0x10,
+        PrintConv => {
+            0x00 => 'Normal Zone',
+            0x10 => 'Wide Zone',
+        },
+    },
+    2.3 => { # CSa3
+        Name => 'AFAssist',
+        Mask => 0x01,
+        PrintConv => { 0x00 => 'On', 0x01 => 'Off' },
+    },
+    2.4 => { # CSa4
+        Name => 'AFPointIllumination',
+        Mask => 0x06,
+        PrintConv => {
+            0x00 => 'Auto',
+            0x02 => 'On',
+            0x04 => 'Off',
+        },
+    },
+    2.5 => { # CSa5
+        Name => 'FocusPointWrap',
+        Mask => 0x08,
+        PrintConv => {
+            0x00 => 'No Wrap',
+            0x08 => 'Wrap',
+        },
+    },
+    3.1 => { # CSa6
+        Name => 'AELockForMB-D80',
+        Mask => 0x1c,
+        PrintConv => {
+            0x00 => 'AE Lock Only',
+            0x04 => 'AF Lock Only',
+            0x08 => 'AE Lock (hold)',
+            0x0c => 'AF-On',
+            0x10 => 'FV Lock',
+            0x14 => 'Focus Point Selection',
+            0x1c => 'AE/AF Lock',
+        },
+    },
+    3.2 => { # CSd12
+        Name => 'MB-D80BatteryType',
+        Mask => 0x03,
+        PrintConv => {
+            0x00 => 'LR6 (AA alkaline)',
+            0x01 => 'HR6 (AA Ni-MH)',
+            0x02 => 'FR6 (AA lithium)',
+            0x03 => 'ZR6 (AA Ni-Mn)',
+        },
+    },
+    4.1 => { # CSd1
+        Name => 'Beep',
+        Mask => 0x40,
+        PrintConv => {
+            0x40 => 'On',
+            0x00 => 'Off',
+        },
+    },
+    4.2 => { # CSd2
+        Name => 'GridDisplay',
+        Mask => 0x02,
+        PrintConv => { 0x00 => 'Off', 0x02 => 'On' },
+    },
+    4.3 => { # CSd3
+        Name => 'ISODisplay',
+        Mask => 0x0c,
+        PrintConv => {
+            0x00 => 'Show ISO/Easy ISO',
+            0x04 => 'Show ISO Sensitivity',
+            0x0c => 'Show Frame Count',
+        },
+    },
+    4.4 => { # CSd4
+        Name => 'ViewfinderWarning',
+        Mask => 0x01,
+        PrintConv => { 0x00 => 'On', 0x01 => 'Off' },
+    },
+    4.5 => { # CSf6
+        Name => 'NoMemoryCard',
+        Mask => 0x20,
+        PrintConv => {
+            0x00 => 'Release Locked',
+            0x20 => 'Enable Release',
+        },
+    },
+    5.1 => { # CSd5
+        Name => 'ScreenTips',
+        Mask => 0x04,
+        PrintConv => { 0x00 => 'Off', 0x04 => 'On' },
+    },
+    5.2 => { # CSd7
+        Name => 'FileNumberSequence',
+        Mask => 0x08,
+        PrintConv => { 0x00 => 'On', 0x08 => 'Off' },
+    },
+    5.3 => { # CSd8
+        Name => 'ShootingInfoDisplay',
+        Mask => 0xc0,
+        PrintConv => {
+            0x00 => 'Auto',
+            0x80 => 'Manual (dark on light)',
+            0xc0 => 'Manual (light on dark)',
+        },
+    },
+    5.4 => { # CSd9
+        Name => 'LCDIllumination',
+        Mask => 0x20,
+        PrintConv => { 0x00 => 'Off', 0x20 => 'On' },
+    },
+    6.1 => { # CSb2
+        Name => 'EasyExposureComp',
+        Mask => 0x01,
+        PrintConv => { 0x00 => 'Off', 0x01 => 'On' },
+    },
+    6.2 => { # CSf7
+        Name => 'ReverseIndicators',
+        Mask => 0x80,
+        PrintConv => {
+            0x00 => '+ 0 -',
+            0x80 => '- 0 +',
+        },
+    },
+    7.1 => { # CSb1
+        Name => 'ExposureControlStepSize',
+        Mask => 0x40,
+        PrintConv => {
+            0x00 => '1/3 EV',
+            0x40 => '1/2 EV',
+        },
+    },
+    8.1 => { # CSb3
+        Name => 'CenterWeightedAreaSize',
+        Mask => 0x60,
+        PrintConv => {
+            0x00 => '6 mm',
+            0x20 => '8 mm',
+            0x40 => '10 mm',
+        },
+    },
+    8.2 => { # CSb4-a
+        Name => 'FineTuneOptMatrixMetering',
+        Mask => 0x0f,
+        ValueConv => '($val > 0x7 ? $val - 0x10 : $val) / 6',
+        ValueConvInv => 'int($val*6+($val>0?0.5:-0.5)) & 0x0f',
+        PrintConv => '$val ? sprintf("%+.2f", $val) : 0',
+        PrintConvInv => 'eval $val',
+    },
+    9.1 => { # CSb4-b
+        Name => 'FineTuneOptCenterWeighted',
+        Mask => 0xf0,
+        ValueConv => '($val > 0x70 ? $val - 0x100 : $val) / 0x60',
+        ValueConvInv => '(int($val*6+($val>0?0.5:-0.5))<<4) & 0xf0',
+        PrintConv => '$val ? sprintf("%+.2f", $val) : 0',
+        PrintConvInv => 'eval $val',
+    },
+    9.2 => { # CSb4-c
+        Name => 'FineTuneOptSpotMetering',
+        Mask => 0x0f,
+        ValueConv => '($val > 0x7 ? $val - 0x10 : $val) / 6',
+        ValueConvInv => 'int($val*6+($val>0?0.5:-0.5)) & 0x0f',
+        PrintConv => '$val ? sprintf("%+.2f", $val) : 0',
+        PrintConvInv => 'eval $val',
+    },
+    11.1 => { # CSd6
+        Name => 'CLModeShootingSpeed',
+        Mask => 0x07,
+        PrintConv => '"$val fps"',
+        PrintConvInv => '$val=~s/\s*fps//i; $val',
+    },
+    11.2 => { # CSd10
+        Name => 'ExposureDelayMode',
+        Mask => 0x40,
+        PrintConv => {
+            0x00 => 'Off',
+            0x40 => 'On',
+        },
+    },
+    13.1 => { # CSe4
+        Name => 'AutoBracketSet',
+        Mask => 0xe0, #(NC)
+        PrintConv => {
+            0x00 => 'AE & Flash', # default
+            0x20 => 'AE Only',
+            0x40 => 'Flash Only', #(NC)
+            0x60 => 'WB Bracketing', #(NC)
+            0x80 => 'Active D-Lighting', #(NC)
+        },
+    },
+    13.2 => { # CSe6
+        Name => 'AutoBracketOrder',
+        Mask => 0x10,
+        PrintConv => {
+            0x00 => '0,-,+',
+            0x10 => '-,0,+',
+        },
+    },
+    14.1 => { # CSf3
+        Name => 'FuncButton',
+        Mask => 0x78,
+        PrintConv => {
+            0x08 => 'Framing Grid',
+            0x10 => 'AF-area Mode',
+            0x18 => 'Center Focus Point',
+            0x20 => 'FV Lock', # default
+            0x28 => 'Flash Off',
+            0x30 => 'Matrix Metering',
+            0x38 => 'Center-weighted Metering',
+            0x40 => 'Spot Metering',
+            0x48 => 'My Menu Top',
+            0x50 => '+ NEF (RAW)',
+        },
+    },
+    16.1 => { # CSf2
+        Name => 'OKButton',
+        Mask => 0x18,
+        PrintConv => {
+            0x08 => 'Select Center Focus Point',
+            0x10 => 'Highlight Active Focus Point',
+            0x18 => 'Not Used', #(NC)
+            0x00 => 'Not Used', #(NC)
+        },
+    },
+    17.1 => { # CSf4
+        Name => 'AELockButton',
+        Mask => 0x38,
+        PrintConv => {
+            0x00 => 'AE/AF Lock',
+            0x08 => 'AE Lock Only',
+            0x10 => 'AF Lock Only', #(NC)
+            0x18 => 'AE Lock (hold)', #(NC)
+            0x20 => 'AF-ON', #(NC)
+            0x28 => 'FV Lock', #(NC)
+        },
+    },
+    18.1 => { # CSf5-a
+        Name => 'CommandDialsReverseRotation',
+        Mask => 0x80,
+        PrintConv => { 0x00 => 'No', 0x80 => 'Yes' },
+    },
+    18.2 => { # CSc1
+        Name => 'ShutterReleaseButtonAE-L',
+        Mask => 0x02,
+        PrintConv => { 0x00 => 'Off', 0x02 => 'On' },
+    },
+    19.1 => { # CSc2
+        Name => 'MeteringTime',
+        Mask => 0xf0,
+        PrintConvColumns => 2,
+        PrintConv => {
+            0x00 => '4 s',
+            0x10 => '6 s', # default
+            0x20 => '8 s',
+            0x30 => '16 s',
+            0x40 => '30 s',
+            0x50 => '1 min',
+            0x60 => '5 min',
+            0x70 => '10 min',
+            0x80 => '30 min',
+        },
+    },
+    19.2 => { # CSc5
+        Name => 'RemoteOnDuration',
+        Mask => 0x03,
+        PrintConv => {
+            0x00 => '1 min',
+            0x01 => '5 min',
+            0x02 => '10 min',
+            0x03 => '15 min',
+        },
+    },
+    20.1 => { # CSc3-a
+        Name => 'SelfTimerTime',
+        Mask => 0xc0,
+        PrintConv => {
+            0x00 => '2 s',
+            0x40 => '5 s',
+            0x80 => '10 s', # default
+            0xc0 => '20 s',
+        },
+    },
+    20.2 => { # CSc3-b
+        Name => 'SelfTimerShotCount',
+        Mask => 0x1e,
+        ValueConv => '$val >> 1',
+        ValueConvInv => '$val << 1',
+    },
+    21.1 => { # CSc4-a
+        Name => 'PlaybackMonitorOffTime',
+        Mask => 0x1c,
+        PrintConv => {
+            0x00 => '4 s',
+            0x04 => '10 s', # default
+            0x08 => '20 s',
+            0x0c => '1 min',
+            0x10 => '5 min',
+            0x14 => '10 min',
+        },
+    },
+    21.2 => { # CSc4-d
+        Name => 'ImageReviewTime',
+        Mask => 0xe0,
+        PrintConv => {
+            0x00 => '4 s', # default
+            0x20 => '10 s',
+            0x40 => '20 s',
+            0x60 => '1 min',
+            0x80 => '5 min',
+            0xa0 => '10 min',
+        },
+    },
+    22.1 => { # CSc4-b
+        Name => 'MenuMonitorOffTime',
+        Mask => 0xe0,
+        PrintConv => {
+            0x00 => '4 s',
+            0x20 => '10 s',
+            0x40 => '20 s', # default
+            0x60 => '1 min',
+            0x80 => '5 min',
+            0xa0 => '10 min',
+        },
+    },
+    22.2 => { # CSc4-c
+        Name => 'ShootingInfoMonitorOffTime',
+        Mask => 0x1c,
+        PrintConv => {
+            0x00 => '4 s',
+            0x04 => '10 s', # default
+            0x08 => '20 s',
+            0x0c => '1 min',
+            0x10 => '5 min',
+            0x14 => '10 min',
+        },
+    },
+    23.1 => { # CSe1
+        Name => 'FlashShutterSpeed',
+        Mask => 0x0f,
+        PrintConvColumns => 2,
+        PrintConv => {
+            0x00 => '1/60 s', # default
+            0x01 => '1/30 s',
+            0x02 => '1/15 s',
+            0x03 => '1/8 s',
+            0x04 => '1/4 s',
+            0x05 => '1/2 s',
+            0x06 => '1 s',
+            0x07 => '2 s',
+            0x08 => '4 s',
+            0x09 => '8 s',
+            0x0a => '15 s',
+            0x0b => '30 s',
+        },
+    },
+    24.1 => { # CSe2-a
+        Name => 'InternalFlash',
+        Mask => 0xc0,
+        PrintConv => {
+            0x00 => 'TTL',
+            0x40 => 'Manual',
+            0x80 => 'Repeating Flash',
+            0xc0 => 'Commander Mode',
+        },
+    },
+    24.2 => { # CSe2-b
+        Name => 'ManualFlashOutput',
+        Mask => 0x1f,
+        ValueConv => '2 ** (-$val/3)',
+        ValueConvInv => '$val > 0 ? -3*log($val)/log(2) : 0',
+        PrintConv => q{
+            return 'Full' if $val > 0.99;
+            Image::ExifTool::Exif::PrintExposureTime($val);
+        },
+        PrintConvInv => '$val=~/F/i ? 1 : eval $val',
+    },
+    25.1 => { # CSe2-ca
+        Name => 'RepeatingFlashOutput',
+        Mask => 0x70,
+        ValueConv => '2 ** (-($val>>4)-2)',
+        ValueConvInv => '$val > 0 ? int(-log($val)/log(2)-2+0.5)<<4 : 0',
+        PrintConv => 'Image::ExifTool::Exif::PrintExposureTime($val)',
+        PrintConvInv => 'eval $val',
+    },
+    25.2 => { # CSe2-cb
+        Name => 'RepeatingFlashCount',
+        Mask => 0x0f,
+        ValueConv => '$val < 10 ? $val + 1 : 5 * ($val - 7)',
+        ValueConvInv => '$val <= 10 ? $val - 1 : $val / 5 + 7',
+    },
+    26.1 => { # CSe2-cc (NC)
+        Name => 'RepeatingFlashRate',
+        Mask => 0xf0,
+        ValueConv => 'my $v=($val>>4); $v < 10 ? $v + 1 : 10 * ($v - 8)',
+        ValueConvInv => 'int(($val <= 10 ? $val - 1 : $val / 10 + 8) + 0.5) << 4',
+        PrintConv => '"$val Hz"',
+        PrintConvInv => '$val=~/(\d+)/; $1 || 0',
+    },
+    31.1 => { # CSd11
+        Name => 'FlashWarning',
+        Mask => 0x80,
+        PrintConv => {
+            0x00 => 'On',
+            0x80 => 'Off',
+        },
+    },
+    31.2 => { # CSe2-ea
+        Name => 'CommanderInternalTTLComp',
+        Mask => 0x1f,
+        ValueConv => '($val - 9) / 3',
+        ValueConvInv => '$val * 3 + 9',
+        PrintConv => '$val ? sprintf("%+.1f",$val) : 0',
+        PrintConvInv => '$val',
+    },
+    31.3 => { # CSe3
+        Name => 'ModelingFlash',
+        Mask => 0x20,
+        PrintConv => { 0x00 => 'On', 0x20 => 'Off' },
+    },
+    31.4 => { # CSe5
+        Name => 'AutoFP',
+        Mask => 0x40,
+        PrintConv => {
+            0x00 => 'Off',
+            0x40 => 'On',
+        },
+    },
+    32.1 => { # CSe2-eb
+        Name => 'CommanderGroupA_TTLComp',
+        Mask => 0x1f,
+        ValueConv => '($val - 9) / 3',
+        ValueConvInv => '$val * 3 + 9',
+        PrintConv => '$val ? sprintf("%+.1f",$val) : 0',
+        PrintConvInv => '$val',
+    },
+    33.1 => { # CSe2-ec
+        Name => 'CommanderGroupB_TTLComp',
+        Mask => 0x1f,
+        ValueConv => '($val - 9) / 3',
+        ValueConvInv => '$val * 3 + 9',
+        PrintConv => '$val ? sprintf("%+.1f",$val) : 0',
+        PrintConvInv => '$val',
+    },
+    34.1 => { # CSa7
+        Name => 'LiveViewAF',
+        Mask => 0xc0,
+        PrintConv => {
+            0x00 => 'Face Priority', #(NC)
+            0x40 => 'Wide Area',
+            0x80 => 'Normal Area',
+        },
+    },
+);
+
 # D300 (ref JD) and D3 (ref 1/PH) custom settings
 %Image::ExifTool::NikonCustom::SettingsD3 = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
@@ -715,7 +1207,23 @@ $VERSION = '1.00';
     #   CSf1-d (D3X,D3S) MultiSelectorLiveView
     #   CSf1 (D300S) LightSwitch
     #   CSf7 (D3S) BracketButton
-    0.1 => { # CSa1
+    0.1 => { #1
+        Name => 'CustomSettingsBank',
+        Mask => 0x03,
+        PrintConv => {
+            0 => 'A',
+            1 => 'B',
+            2 => 'C',
+            3 => 'D',
+        },
+    },
+    0.2 => { #1
+        Name => 'CustomSettingsAllDefault',
+        Notes => '"No" if any custom setting for this bank was changed from the default',
+        Mask => 0x80,
+        PrintConv => { 0x00 => 'Yes', 0x80 => 'No' },
+    },
+    1.1 => { # CSa1
         Name => 'AF-CPrioritySelection',
         Mask => 0xc0,
         PrintConv => {
@@ -724,7 +1232,7 @@ $VERSION = '1.00';
             0x80 => 'Focus',
         },
     },
-    0.2 => { # CSa2
+    1.2 => { # CSa2
         Name => 'AF-SPrioritySelection',
         Mask => 0x20,
         PrintConv => {
@@ -732,7 +1240,7 @@ $VERSION = '1.00';
             0x20 => 'Release',
         },
     },
-    0.3 => { # CSa8
+    1.3 => { # CSa8
         Name => 'AFPointSelection',
         Mask => 0x10,
         PrintConv => {
@@ -740,7 +1248,7 @@ $VERSION = '1.00';
             0x10 => '11 Points',
         },
     },
-    0.4 => { # CSa3
+    1.4 => { # CSa3
         Name => 'DynamicAFArea',
         Mask => 0x0c,
         PrintConv => {
@@ -750,7 +1258,7 @@ $VERSION = '1.00';
             0x0c => '51 Points (3D-tracking)',
         },
     },
-    0.5 => { # CSa4
+    1.5 => { # CSa4
         Name => 'FocusTrackingLockOn',
         Mask => 0x03,
         PrintConv => {
@@ -760,7 +1268,7 @@ $VERSION = '1.00';
             0x03 => 'Off',
         },
     },
-    1.1 => { # CSa5
+    2.1 => { # CSa5
         Name => 'AFActivation',
         Mask => 0x80,
         PrintConv => {
@@ -768,7 +1276,7 @@ $VERSION = '1.00';
             0x80 => 'AF-On Only',
         },
     },
-    1.2 => { # CSa7
+    2.2 => { # CSa7
         Name => 'FocusPointWrap',
         Mask => 0x08,
         PrintConv => {
@@ -776,7 +1284,7 @@ $VERSION = '1.00';
             0x08 => 'Wrap',
         },
     },
-    1.3 => [ # CSa6
+    2.3 => [ # CSa6
         {
             Name => 'AFPointIllumination',
             Condition => '$$self{Model} =~ /D3[SX]?\b/',
@@ -800,7 +1308,7 @@ $VERSION = '1.00';
             },
         },
     ],
-    1.4 => { # CSa6-b (D3, added by firmware update)
+    2.4 => { # CSa6-b (D3, added by firmware update)
         Name => 'AFPointBrightness',
         Condition => '$$self{Model} =~ /D3[SX]?\b/',
         Notes => 'D3 only',
@@ -812,14 +1320,14 @@ $VERSION = '1.00';
             0x06 => 'Extra High',
         },
     },
-    1.5 => { # CSa9 (D300)
+    2.5 => { # CSa9 (D300)
         Name => 'AFAssist',
         Condition => '$$self{Model} =~ /D300S?\b/',
         Notes => 'D300 only',
         Mask => 0x01,
         PrintConv => { 0x00 => 'On', 0x01 => 'Off' },
     },
-    2.1 => { # CSa9 (D3)
+    3.1 => { # CSa9 (D3)
         Name => 'AFOnButton',
         Condition => '$$self{Model} =~ /D3[SX]?\b/',
         Notes => 'D3 only',
@@ -833,7 +1341,7 @@ $VERSION = '1.00';
             5 => 'AF Lock Only',
         },
     },
-    2.2 => { # CSa10 (D3)
+    3.2 => { # CSa10 (D3)
         Name => 'VerticalAFOnButton',
         Condition => '$$self{Model} =~ /D3[SX]?\b/',
         Notes => 'D3 only',
@@ -848,7 +1356,7 @@ $VERSION = '1.00';
             0x70 => 'Same as AF On',
         },
     },
-    2.3 => { # CSa10 (D300)
+    3.3 => { # CSa10 (D300)
         Name => 'AF-OnForMB-D10',
         Condition => '$$self{Model} =~ /D300S?\b/',
         Notes => 'D300 only',
@@ -863,7 +1371,7 @@ $VERSION = '1.00';
             0x60 => 'Same as FUNC Button',
         },
     },
-    5.1 => { # CSb1
+    6.1 => { # CSb1
         Name => 'ISOStepSize',
         Mask => 0xc0,
         PrintConv => {
@@ -872,7 +1380,7 @@ $VERSION = '1.00';
             0x80 => '1 EV',
         },
     },
-    5.2 => { # CSb2
+    6.2 => { # CSb2
         Name => 'ExposureControlStepSize',
         Mask => 0x30,
         PrintConv => {
@@ -881,7 +1389,7 @@ $VERSION = '1.00';
             0x20 => '1 EV',
         },
     },
-    5.3 => { # CSb3
+    6.3 => { # CSb3
         Name => 'ExposureCompStepSize',
         Mask => 0x0c,
         PrintConv => {
@@ -890,7 +1398,7 @@ $VERSION = '1.00';
             0x08 => '1 EV',
         },
     },
-    5.4 => { # CSb4
+    6.4 => { # CSb4
         Name => 'EasyExposureCompensation',
         Mask => 0x03,
         PrintConv => {
@@ -899,7 +1407,7 @@ $VERSION = '1.00';
             0x02 => 'On (auto reset)',
         },
     },
-    6.1 => [ # CSb5
+    7.1 => [ # CSb5
         {
             Name => 'CenterWeightedAreaSize',
             Condition => '$$self{Model} =~ /D3[SX]?\b/',
@@ -926,31 +1434,31 @@ $VERSION = '1.00';
             },
         },
     ],
-    6.2 => { # CSb6-b
+    7.2 => { # CSb6-b
         Name => 'FineTuneOptCenterWeighted',
         Mask => 0x0f,
         ValueConv => '($val > 0x7 ? $val - 0x10 : $val) / 6',
         ValueConvInv => 'int($val*6+($val>0?0.5:-0.5)) & 0x0f',
-        PrintConv => '$val ? sprintf("%+.1f", $val) : 0',
+        PrintConv => '$val ? sprintf("%+.2f", $val) : 0',
         PrintConvInv => 'eval $val',
     },
-    7.1 => { # CSb6-a
+    8.1 => { # CSb6-a
         Name => 'FineTuneOptMatrixMetering',
         Mask => 0xf0,
         ValueConv => '($val > 0x70 ? $val - 0x100 : $val) / 0x60',
         ValueConvInv => '(int($val*6+($val>0?0.5:-0.5))<<4) & 0xf0',
-        PrintConv => '$val ? sprintf("%+.1f", $val) : 0',
+        PrintConv => '$val ? sprintf("%+.2f", $val) : 0',
         PrintConvInv => 'eval $val',
     },
-    7.2 => { # CSb6-c
+    8.2 => { # CSb6-c
         Name => 'FineTuneOptSpotMetering',
         Mask => 0x0f,
         ValueConv => '($val > 0x7 ? $val - 0x10 : $val) / 6',
         ValueConvInv => 'int($val*6+($val>0?0.5:-0.5)) & 0x0f',
-        PrintConv => '$val ? sprintf("%+.1f", $val) : 0',
+        PrintConv => '$val ? sprintf("%+.2f", $val) : 0',
         PrintConvInv => 'eval $val',
     },
-    8.1 => { # CSf1-a, CSf2-a (D300S)
+    9.1 => { # CSf1-a, CSf2-a (D300S)
         Name => 'MultiSelectorShootMode',
         Mask => 0xc0,
         PrintConv => {
@@ -959,7 +1467,7 @@ $VERSION = '1.00';
             0x80 => 'Not Used',
         },
     },
-    8.2 => { # CSf1-b, CSf2-b (D300S)
+    9.2 => { # CSf1-b, CSf2-b (D300S)
         Name => 'MultiSelectorPlaybackMode',
         Mask => 0x30,
         PrintConv => {
@@ -969,7 +1477,7 @@ $VERSION = '1.00';
             0x30 => 'Choose Folder',
         },
     },
-    8.3 => [ # CSf1-c, CSf2-c (D300S)
+    9.3 => [ # CSf1-c, CSf2-c (D300S)
         {
             Name => 'InitialZoomSetting',
             Condition => '$$self{Model} =~ /D3[SX]?\b/',
@@ -992,7 +1500,7 @@ $VERSION = '1.00';
             },
         },
     ],
-    8.4 => { # CSf2 (D300,D3), CSf3 (D300S)
+    9.4 => { # CSf2 (D300,D3), CSf3 (D300S)
         Name => 'MultiSelector',
         Mask => 0x01,
         PrintConv => {
@@ -1000,18 +1508,18 @@ $VERSION = '1.00';
             0x01 => 'Reset Meter-off Delay',
         },
     },
-    9.1 => { # CSd9 (D300,D3S), CSd10 (D300S), CSd8 (D3)
+    10.1 => { # CSd9 (D300,D3S), CSd10 (D300S), CSd8 (D3)
         Name => 'ExposureDelayMode',
         Mask => 0x40,
         PrintConv => { 0x00 => 'Off', 0x40 => 'On' },
     },
-    9.2 => { # CSd4 (D300), CDs5 (D300S), CSd2-a (D3)
+    10.2 => { # CSd4 (D300), CDs5 (D300S), CSd2-a (D3)
         Name => 'CLModeShootingSpeed',
         Mask => 0x07,
         PrintConv => '"$val fps"',
         PrintConvInv => '$val=~s/\s*fps//i; $val',
     },
-    9.3 => { # (D3 CSd2-b)
+    10.3 => { # (D3 CSd2-b)
         Name => 'CHModeShootingSpeed',
         Condition => '$$self{Model} =~ /D3[SX]?\b/',
         Notes => 'D3 only',
@@ -1022,11 +1530,11 @@ $VERSION = '1.00';
             0x20 => '11 fps',
         },
     },
-    10 => { # CSd5 (D300), CSd6 (D300S), CSd3 (D3)
+    11 => { # CSd5 (D300), CSd6 (D300S), CSd3 (D3)
         Name => 'MaxContinuousRelease',
         # values: 1-100 (D300), 1-130 (D3)
     },
-    11.1 => { # CSf10, CSf11 (D3S,D300S)
+    12.1 => { # CSf10, CSf11 (D3S,D300S)
         Name => 'ReverseIndicators',
         Mask => 0x20,
         PrintConv => {
@@ -1034,7 +1542,7 @@ $VERSION = '1.00';
             0x20 => '- 0 +',
         },
     },
-    11.2 => [ # CSd6 (D300), CSd7 (D300S), CSd4 (D3)
+    12.2 => [ # CSd6 (D300), CSd7 (D300S), CSd4 (D3)
         {
             Name => 'FileNumberSequence',
             Condition => '$$self{Model} =~ /D3[SX]?\b/',
@@ -1049,7 +1557,7 @@ $VERSION = '1.00';
             PrintConv => { 0x00 => 'On', 0x08 => 'Off' },
         },
     ],
-    11.3 => { # CSd5-a (D3)
+    12.3 => { # CSd5-a (D3)
         Name => 'RearDisplay',
         Condition => '$$self{Model} =~ /D3[SX]?\b/',
         Notes => 'D3 only',
@@ -1059,7 +1567,7 @@ $VERSION = '1.00';
             0x80 => 'Exposures Remaining',
         },
     },
-    11.4 => { # CSd5-b (D3)
+    12.4 => { # CSd5-b (D3)
         Name => 'ViewfinderDisplay',
         Condition => '$$self{Model} =~ /D3[SX]?\b/',
         Notes => 'D3 only',
@@ -1069,7 +1577,7 @@ $VERSION = '1.00';
             0x40 => 'Exposures Remaining',
         },
     },
-    11.5 => { # CSd11 (D300), CSd12 (D300S)
+    12.5 => { # CSd11 (D300), CSd12 (D300S)
         Name => 'BatteryOrder',
         Condition => '$$self{Model} =~ /D300S?\b/',
         Notes => 'D300 only',
@@ -1079,7 +1587,7 @@ $VERSION = '1.00';
             0x04 => 'Camera Battery First',
         },
     },
-    11.6 => { # CSd10 (D300), CSd11 (D300S)
+    12.6 => { # CSd10 (D300), CSd11 (D300S)
         Name => 'MB-D10Batteries',
         Condition => '$$self{Model} =~ /D300S?\b/',
         Notes => 'D300 only',
@@ -1091,7 +1599,7 @@ $VERSION = '1.00';
             0x03 => 'ZR6 (AA Ni-Mn)',
         },
     },
-    12.1 => { # CSd1
+    13.1 => { # CSd1
         Name => 'Beep',
         Mask => 0xc0,
         PrintConv => {
@@ -1100,7 +1608,7 @@ $VERSION = '1.00';
             0x80 => 'Off',
         },
     },
-    12.2 => { # CSd7 (D300), CSd8 (D300S), CSd6 (D3)
+    13.2 => { # CSd7 (D300), CSd8 (D300S), CSd6 (D3)
         Name => 'ShootingInfoDisplay',
         Mask => 0x30,
         PrintConv => {
@@ -1110,21 +1618,21 @@ $VERSION = '1.00';
             0x30 => 'Manual (light on dark)',
         },
     },
-    12.3 => { # CSd2 (D300)
+    13.3 => { # CSd2 (D300)
         Name => 'GridDisplay',
         Condition => '$$self{Model} =~ /D300S?\b/',
         Notes => 'D300 only',
         Mask => 0x02,
         PrintConv => { 0x00 => 'Off', 0x02 => 'On' },
     },
-    12.4 => { # CSd3 (D300)
+    13.4 => { # CSd3 (D300)
         Name => 'ViewfinderWarning',
         Condition => '$$self{Model} =~ /D300S?\b/',
         Notes => 'D300 only',
         Mask => 0x01,
         PrintConv => { 0x00 => 'On', 0x01 => 'Off' },
     },
-    13.1 => [ # CSf5-a (ref 1), CSf6-a (D300S)
+    14.1 => [ # CSf5-a (ref 1), CSf6-a (D300S)
         {
             Name => 'PreviewButton',
             Condition => '$$self{Model} =~ /D3[SX]?\b/',
@@ -1176,7 +1684,7 @@ $VERSION = '1.00';
             },
         },
     ],
-    13.2 => [ # CSf5-b (PH,NC), CSf6-b (D300S)
+    14.2 => [ # CSf5-b (PH,NC), CSf6-b (D300S)
         {
             Name => 'PreviewButtonPlusDials',
             Condition => '$$self{Model} =~ /D3[SX]?\b/',
@@ -1206,7 +1714,7 @@ $VERSION = '1.00';
             },
         },
     ],
-    14.1 => [ # CSf4-a (ref 1), CSf5-a (D300S)
+    15.1 => [ # CSf4-a (ref 1), CSf5-a (D300S)
         {
             Name => 'FuncButton',
             Condition => '$$self{Model} =~ /D3[SX]?\b/',
@@ -1258,7 +1766,7 @@ $VERSION = '1.00';
             },
         },
     ],
-    14.2 => [ # CSf4-b (ref 1), CSf5-b (D300S)
+    15.2 => [ # CSf4-b (ref 1), CSf5-b (D300S)
         {
             Name => 'FuncButtonPlusDials',
             Condition => '$$self{Model} =~ /D3[SX]?\b/',
@@ -1288,7 +1796,7 @@ $VERSION = '1.00';
             },
         },
     ],
-    15.1 => [ # CSf6-a (ref 1), CSf7-a (D300S)
+    16.1 => [ # CSf6-a (ref 1), CSf7-a (D300S)
         {
             Name => 'AELockButton',
             Condition => '$$self{Model} =~ /D3[SX]?\b/',
@@ -1342,7 +1850,7 @@ $VERSION = '1.00';
             },
         },
     ],
-    15.2 => [ # CSf6-b (ref 1), CSf7-b (D300S)
+    16.2 => [ # CSf6-b (ref 1), CSf7-b (D300S)
         {
             Name => 'AELockButtonPlusDials',
             Condition => '$$self{Model} =~ /D3[SX]?\b/',
@@ -1372,17 +1880,17 @@ $VERSION = '1.00';
             },
         },
     ],
-    16.1 => { # CSf7-a, CSf8-a (D3S,D300S)
+    17.1 => { # CSf7-a, CSf8-a (D3S,D300S)
         Name => 'CommandDialsReverseRotation',
         Mask => 0x80,
         PrintConv => { 0x00 => 'No', 0x80 => 'Yes' },
     },
-    16.2 => { # CSf7-b, CSf8-b (D3S,D300S)
+    17.2 => { # CSf7-b, CSf8-b (D3S,D300S)
         Name => 'CommandDialsChangeMainSub',
         Mask => 0x40,
         PrintConv => { 0x00 => 'Off', 0x40 => 'On' },
     },
-    16.3 => { # CSf7-c, CSf8-c (D3S,D300S)
+    17.3 => { # CSf7-c, CSf8-c (D3S,D300S)
         Name => 'CommandDialsApertureSetting',
         Mask => 0x20,
         PrintConv => {
@@ -1390,17 +1898,17 @@ $VERSION = '1.00';
             0x20 => 'Aperture Ring',
         },
     },
-    16.4 => { # CSf7-d, CSf8-d (D3S,D300S)
+    17.4 => { # CSf7-d, CSf8-d (D3S,D300S)
         Name => 'CommandDialsMenuAndPlayback',
         Mask => 0x10,
         PrintConv => { 0x00 => 'Off', 0x10 => 'On' },
     },
-    16.5 => { # CSd8 (D300,D3S), CSd9 (D300S), CSd7 (D3)
+    17.5 => { # CSd8 (D300,D3S), CSd9 (D300S), CSd7 (D3)
         Name => 'LCDIllumination',
         Mask => 0x08,
         PrintConv => { 0x00 => 'Off', 0x08 => 'On' },
     },
-    16.6 => { # CSf3, CSf4 (D300S)
+    17.6 => { # CSf3, CSf4 (D300S)
         Name => 'PhotoInfoPlayback',
         Mask => 0x04,
         PrintConv => {
@@ -1408,17 +1916,17 @@ $VERSION = '1.00';
             0x04 => 'Info Left-right, Playback Up-down',
         },
     },
-    16.7 => { # CSc1
+    17.7 => { # CSc1
         Name => 'ShutterReleaseButtonAE-L',
         Mask => 0x02,
         PrintConv => { 0x00 => 'Off', 0x02 => 'On' },
     },
-    16.8 => { # CSf8, CSf9 (D3S,D300S)
+    17.8 => { # CSf8, CSf9 (D3S,D300S)
         Name => 'ReleaseButtonToUseDial',
         Mask => 0x01,
         PrintConv => { 0x00 => 'No', 0x01 => 'Yes' },
     },
-    17.1 => { # CSc3
+    18.1 => { # CSc3
         Name => 'SelfTimerTime',
         Mask => 0x18,
         PrintConv => {
@@ -1428,7 +1936,7 @@ $VERSION = '1.00';
             0x18 => '20 s',
         },
     },
-    17.2 => { # CSc4
+    18.2 => { # CSc4
         Name => 'MonitorOffTime',
         # NOTE: The D3S and D300S have separate settings for Playback,
         # Image Review, Menus, and Information Display times, but these
@@ -1442,21 +1950,21 @@ $VERSION = '1.00';
             0x04 => '10 min',
         },
     },
-    19.1 => [ # CSe1
+    20.1 => [ # CSe1
         {
             Name => 'FlashSyncSpeed',
             Condition => '$$self{Model} =~ /D3[SX]?\b/',
             Notes => 'D3',
-            Mask => 0xf0,
+            Mask => 0xe0,
             PrintConv => {
                 0x00 => '1/250 s (auto FP)',
-                0x10 => '1/250 s',
-                0x20 => '1/200 s',
-                0x30 => '1/160 s',
-                0x40 => '1/125 s',
-                0x50 => '1/100 s',
-                0x60 => '1/80 s',
-                0x70 => '1/60 s',
+                0x20 => '1/250 s',
+                0x40 => '1/200 s',
+                0x60 => '1/160 s',
+                0x80 => '1/125 s',
+                0xa0 => '1/100 s',
+                0xc0 => '1/80 s',
+                0xe0 => '1/60 s',
             },
         },
         {
@@ -1476,7 +1984,7 @@ $VERSION = '1.00';
             },
         },
     ],
-    19.2 => { # CSe2
+    20.2 => { # CSe2
         Name => 'FlashShutterSpeed',
         Mask => 0x0f,
         PrintConvColumns => 2,
@@ -1495,7 +2003,7 @@ $VERSION = '1.00';
             0x0b => '30 s',
         },
     },
-    20.1 => { # CSe5 (D300), CSe4 (D3)
+    21.1 => { # CSe5 (D300), CSe4 (D3)
         Name => 'AutoBracketSet',
         Mask => 0xc0,
         PrintConv => {
@@ -1506,7 +2014,7 @@ $VERSION = '1.00';
             # NOTE: D3S/D300S have an "ADL Bracketing" setting not decoded - PH
         },
     },
-    20.2 => { # CSe6 (D300), CSe5 (D3)
+    21.2 => { # CSe6 (D300), CSe5 (D3)
         Name => 'AutoBracketModeM',
         Mask => 0x30,
         PrintConv => {
@@ -1516,7 +2024,7 @@ $VERSION = '1.00';
             0x30 => 'Flash Only',
         },
     },
-    20.3 => { # CSe7 (D300), CSe6 (D3)
+    21.3 => { # CSe7 (D300), CSe6 (D3)
         Name => 'AutoBracketOrder',
         Mask => 0x08,
         PrintConv => {
@@ -1524,12 +2032,12 @@ $VERSION = '1.00';
             0x08 => '-,0,+',
         },
     },
-    20.4 => { # CSe4 (D300), CSe3 (D3)
+    21.4 => { # CSe4 (D300), CSe3 (D3)
         Name => 'ModelingFlash',
         Mask => 0x01,
         PrintConv => { 0x00 => 'On', 0x01 => 'Off' },
     },
-    21.1 => { # CSf9, CSf10 (D3S,D300S)
+    22.1 => { # CSf9, CSf10 (D3S,D300S)
         Name => 'NoMemoryCard',
         Mask => 0x80,
         PrintConv => {
@@ -1537,7 +2045,7 @@ $VERSION = '1.00';
             0x80 => 'Enable Release',
         },
     },
-    21.2 => { # CSc2
+    22.2 => { # CSc2
         Name => 'MeteringTime',
         Mask => 0x0f,
         PrintConvColumns => 2,
@@ -1554,7 +2062,7 @@ $VERSION = '1.00';
             0x09 => 'No Limit',
         },
     },
-    22.1 => { # CSe3
+    23.1 => { # CSe3
         Name => 'InternalFlash',
         Mask => 0xc0,
         PrintConv => {
@@ -1581,7 +2089,7 @@ $VERSION = '1.00';
         Name => 'AFAreaModeSetting',
         Mask => 0x60,
         PrintConv => {
-            0x00 => 'Single Point',
+            0x00 => 'Single Area',
             0x20 => 'Dynamic Area',
             0x40 => 'Auto-area',
             0x60 => '3D-tracking (11 points)',
@@ -1628,7 +2136,7 @@ $VERSION = '1.00';
             0x20 => 'Enable Release',
         },
     },
-    3 => { # CSd4
+    3.1 => { # CSd4
         Name => 'FileNumberSequence',
         Mask => 0x08,
         PrintConv => { 0x00 => 'On', 0x08 => 'Off' },
@@ -1654,7 +2162,7 @@ $VERSION = '1.00';
             0x80 => '- 0 +',
         },
     },
-    5 => { # CSb1
+    5.1 => { # CSb1
         Name => 'EVStepSize',
         Mask => 0x40,
         PrintConv => {
@@ -1662,7 +2170,7 @@ $VERSION = '1.00';
             0x40 => '1/2 EV',
         },
     },
-    9 => { # CSd5
+    9.1 => { # CSd5
         Name => 'ExposureDelayMode',
         Mask => 0x40,
         PrintConv => {
@@ -1670,7 +2178,7 @@ $VERSION = '1.00';
             0x40 => 'On',
         },
     },
-    11 => { # CSe2
+    11.1 => { # CSe2
         Name => 'AutoBracketSet',
         Mask => 0xc0,
         PrintConv => {
@@ -1679,7 +2187,7 @@ $VERSION = '1.00';
             0x80 => 'WB Bracketing',
         },
     },
-    12 => { # CSf1
+    12.1 => { # CSf1
         Name => 'TimerFunctionButton',
         Mask => 0x38,
         PrintConv => {
@@ -1693,7 +2201,7 @@ $VERSION = '1.00';
             0x38 => 'Auto Bracketing',
         },
     },
-    15 => { # CSf2
+    15.1 => { # CSf2
         Name => 'AELockButton',
         Mask => 0x38,
         PrintConv => {
@@ -1710,7 +2218,7 @@ $VERSION = '1.00';
         PrintConv => { 0x00 => 'Off', 0x02 => 'On' },
     },
     16.2 => { # CSf3
-        Name => 'CommandDialReverseRotation',
+        Name => 'CommandDialsReverseRotation',
         Mask => 0x80,
         PrintConv => { 0x00 => 'No', 0x80 => 'Yes' },
     },
@@ -1751,7 +2259,7 @@ $VERSION = '1.00';
         ValueConv => '$val >> 1',
         ValueConvInv => '$val << 1',
     },
-    19 => { # CSc2-b
+    19.1 => { # CSc2-b
         Name => 'ImageReviewTime',
         Mask => 0xe0,
         PrintConv => {
@@ -1762,7 +2270,7 @@ $VERSION = '1.00';
             0x80 => '10 min',
         },
     },
-    20 => { # CSc2-a
+    20.1 => { # CSc2-a
         Name => 'PlaybackMenusTime',
         Mask => 0xe0,
         PrintConv => {
@@ -1783,7 +2291,7 @@ $VERSION = '1.00';
     },
     22.2 => { # CSe1-b
         Name => 'ManualFlashOutput',
-        Mask => 0x0f,
+        Mask => 0x1f,
         ValueConv => '2 ** (-$val/3)',
         ValueConvInv => '$val > 0 ? -3*log($val)/log(2) : 0',
         PrintConv => q{
@@ -1792,7 +2300,7 @@ $VERSION = '1.00';
         },
         PrintConvInv => '$val=~/F/i ? 1 : eval $val',
     },
-    32 => { # CSa3
+    32.1 => { # CSa3
         Name => 'LiveViewAF',
         Mask => 0x60,
         PrintConv => {
@@ -1827,7 +2335,7 @@ this information.
 
 =head1 AUTHOR
 
-Copyright 2003-2009, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
