@@ -906,7 +906,7 @@ sub ProcessChunks($$$)
                     $start += $base;
                     #### eval Base ($start)
                     $newBase = eval $newBase;
-                    $baseShift = $newBase - $$dirInfo{Base};
+                    $baseShift = $newBase - $base;
                     $start -= $base;
                 }
             } elsif (not $$tagInfo{Binary}) {
@@ -1015,7 +1015,7 @@ including Windows WAV audio and AVI video files.
 
 =head1 AUTHOR
 
-Copyright 2003-2009, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
