@@ -512,8 +512,7 @@ sub CheckXMP($$$)
         if ($exifTool->{OPTIONS}->{Charset} ne 'UTF8') {
             if ($$valPtr =~ /[\x80-\xff]/) {
                 # convert from Charset to UTF-8
-                my $val = $exifTool->Charset2Unicode($$valPtr, 'MM');
-                $$valPtr = Image::ExifTool::Unicode2UTF8($val,'n');
+                $$valPtr = $exifTool->Encode($$valPtr,'UTF8');
             }
         } else {
             # translate invalid XML characters to "."
@@ -1578,7 +1577,7 @@ This file contains routines to write XMP metadata.
 
 =head1 AUTHOR
 
-Copyright 2003-2009, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
